searchData={"items":[{"type":"module","title":"Tornex.API","doc":"Core API functionality.\n\n`Tornex.API` provides the core Torn API-related functionality to make the API calls.","ref":"Tornex.API.html"},{"type":"module","title":"Configuration - Tornex.API","doc":"Tornex uses compile-time configuration for certain API-related functionality:\n- `base_url` (default: \"https://api.torn.com\") => base of the URL the remainder of the request URL is built off of\n- `comment` (default: \"tex/0.3.0\") => Comment used in the API calls\n- `client` (default: `Tornex.HTTP.FinchClient`) => HTTP client to use in API calls\n\nTo customize one of these, include the following in a `config/*.exs`:\n\n    config :tornex,\n      base_url: \"https://api.torn.com\",\n      comment: \"Tornium\",","ref":"Tornex.API.html#module-configuration"},{"type":"type","title":"Tornex.API.error/0","doc":"","ref":"Tornex.API.html#t:error/0"},{"type":"function","title":"Tornex.API.get/1","doc":"Performs a blocking HTTP GET request against the Torn API for a `Tornex.Query` or `Tornex.SpecQuery`.","ref":"Tornex.API.html#get/1"},{"type":"function","title":"Examples - Tornex.API.get/1","doc":"iex> query = %Tornex.Query{resource: \"user\", resource_id: 1, key: \"\", key_owner: 1, nice: 0}\n    iex> Tornex.API.get(query)\n    ...","ref":"Tornex.API.html#get/1-examples"},{"type":"function","title":"Tornex.API.query_to_url/1","doc":"Converts a `Tornex.Query` or `Tornex.SpecQuery` to the URL required to make the HTTP request.\n\n    iex> query = %Tornex.Query{\n    ...>   resource: \"user\",\n    ...>   resource_id: 1,\n    ...>   key: \"apikey\",\n    ...>   key_owner: \"apikey\",\n    ...>   nice: 10,\n    ...>   selections: [\"basic\", \"discord\"]\n    ...> }\n    iex> Tornex.API.query_to_url(query)\n    \"https://api.torn.com/user/1?selections=basic,discord&key=apikey&comment=tex-#{Mix.Project.config()[:version]}\"\n\n    iex> spec_query =\n    ...>   Tornex.SpecQuery.new()\n    ...>   |> Tornex.SpecQuery.put_path(Torngen.Client.Path.Faction.Id.Basic)\n    ...>   |> Tornex.SpecQuery.put_path(Torngen.Client.Path.Faction.Id.Chains)\n    ...>   |> Tornex.SpecQuery.put_parameter(:id, 89)\n    ...>   |> Tornex.SpecQuery.put_parameter(:limit, 100)\n    ...>   |> Tornex.SpecQuery.put_key(\"apikey\")\n    iex> Tornex.API.query_to_url(spec_query)\n    \"https://api.torn.com/v2/faction/89/?selections=chains,basic&limit=100&comment=tex-#{Mix.Project.config()[:version]}\"","ref":"Tornex.API.html#query_to_url/1"},{"type":"type","title":"Tornex.API.return/0","doc":"","ref":"Tornex.API.html#t:return/0"},{"type":"function","title":"Tornex.API.torn_get/1","doc":"Performs a blocking HTTP GET request against the Torn API for a `Tornex.Query` or `Tornex.SpecQuery`.","ref":"Tornex.API.html#torn_get/1"},{"type":"function","title":"Examples - Tornex.API.torn_get/1","doc":"iex> query = %Tornex.Query{resource: \"user\", resource_id: 1, key: \"\", key_owner: 1, nice: 0}\n    iex> Tornex.API.torn_get(query)\n    ...","ref":"Tornex.API.html#torn_get/1-examples"},{"type":"behaviour","title":"Tornex.HTTP.Client","doc":"The behaviours of HTTP client implementations for `Tornex.API.get/1`.\n\nUnless otherwise configured with `config :tornex, client: AlternateClient`, the default HTTP client is\n`Tornex.HTTP.FinchClient`.\n\nThe `child_spec/1` callback is optional and should be configured for HTTP libraries that require a \nsupervision tree.","ref":"Tornex.HTTP.Client.html"},{"type":"type","title":"Tornex.HTTP.Client.body/0","doc":"Body of the HTTP response.","ref":"Tornex.HTTP.Client.html#t:body/0"},{"type":"callback","title":"Tornex.HTTP.Client.child_spec/1","doc":"Optionally get a child spec to start the HTTP client.","ref":"Tornex.HTTP.Client.html#c:child_spec/1"},{"type":"callback","title":"Tornex.HTTP.Client.get/2","doc":"Perform an HTTP GET to the specified URL with the headers.","ref":"Tornex.HTTP.Client.html#c:get/2"},{"type":"type","title":"Tornex.HTTP.Client.headers/0","doc":"HTTP headers for the HTTP request or response.","ref":"Tornex.HTTP.Client.html#t:headers/0"},{"type":"type","title":"Tornex.HTTP.Client.response/0","doc":"","ref":"Tornex.HTTP.Client.html#t:response/0"},{"type":"type","title":"Tornex.HTTP.Client.status/0","doc":"HTTP status code of the HTTP request.","ref":"Tornex.HTTP.Client.html#t:status/0"},{"type":"callback","title":"Tornex.HTTP.Client.version/0","doc":"Get the name of the library and its version.","ref":"Tornex.HTTP.Client.html#c:version/0"},{"type":"module","title":"Tornex.HTTP.FinchClient","doc":"The built-in HTTP client for `Tornex.API.get/1`.\n\nIf you wish to use the built-in client, add [Finch](https://github.com/sneako/finch) to your dependencies. You can \ncreate your own adapter for an HTTP client by implementing the behaviours defined in `Tornex.HTTP.Client`.","ref":"Tornex.HTTP.FinchClient.html"},{"type":"module","title":"Tornex.PromExPlugin","doc":"PromEx plugin for Tornex to export events and metrics to Prometheus.\n\nTo add `Tornex.PromExPlugin` to your application's PromEx implementation, add `Tornex.PromExPlugin` to your list of plugins:\n\n    @impl true\n    def plugins do\n      [\n        Plugins.Application,\n        Plugins.Beam,\n        Tornex.PromExPlugin\n      ]\n    end\n\nFor more information on setting up PromEx, see the [PromEx docs](https://hexdocs.pm/prom_ex/PromEx.html).","ref":"Tornex.PromExPlugin.html"},{"type":"module","title":"Tornex.Query","doc":"The query struct containing API request data.\n\nThe `Tornex.Query` struct is utilized by `Tornex.API.get/1` and `Tornex.Scheduler.Bucket.enqueue/1` to make API calls. `Tornex.Query` stores the required information to perform the API call.\n\n> #### Tip {: .tip}\n>\n> It is recommended to use `Tornex.SpecQuery` for APIv2 requests as it uses the OpenAPI specification to generate types and to parse and validate API responses. However, APIv2 does not currently support all of the selections that APIv1 supports, so you may have to remain on v1 for the immediate future.","ref":"Tornex.Query.html"},{"type":"function","title":"Tornex.Query.query_priority/1","doc":"Determines the priority of a `Tornex.Query` by the `nice` value as a priority \"bucket\" atom.\n\nThe nice value of a query is based on the Linux niceness value between -20 and 19. A niceness of -20 is the highest priority request and 19 is the lowest priority. For more information on the scheduling of requests against `Tornex.Query`, see `Tornex.Scheduler.Bucket`.\n\nThere are three priority buckets:\n- generic requests (nice > 0) => Requests that are intended to be made with as much latency as needed\n- high priority requests (-10 < nice <= 0) => Requests where latency matters\n- user requests (nice <= -10) => Requests that should be performed immediately, typically initiated by a user action","ref":"Tornex.Query.html#query_priority/1"},{"type":"type","title":"Tornex.Query.t/0","doc":"","ref":"Tornex.Query.html#t:t/0"},{"type":"module","title":"Tornex.Scheduler.Bucket","doc":"Representation of a user's bucket for API requests used for rate-limiting and prioritization.\n\nA bucket can contain as many API requests as the BEAM VM and the memory can handle. However, each bucket will only process 10 API requests every 6 seconds.\n\nThe bucket provides the `enqueue/1` and `enqueue/2` operations providing the core functionality of this module:\n- creating the bucket if necessary,\n- handling telemetry,\n- and request handling.","ref":"Tornex.Scheduler.Bucket.html"},{"type":"module","title":"Bucket Creation - Tornex.Scheduler.Bucket","doc":"Creating buckets is not required, but you can create and store buckets in a different manner if you need to do so.\n\nA buckets can be created with the following two methods:\n  - `new/1` to create a bucket and store the PID of the GenServer manually\n  - `enqueue/1` to create a bucket and let Tornex store the PID of the GenServer in a pre-initialized registry `Tornex.Scheduler.BucketRegistry`","ref":"Tornex.Scheduler.Bucket.html#module-bucket-creation"},{"type":"module","title":"Making API Requests - Tornex.Scheduler.Bucket","doc":"API requests are made with the `enqueue/1` and the `enqueue/2` operations with a `Tornex.Query` or `Tornex.SpecQuery` struct. The operations will then wait until the API call has been scheduled and the Torn API has responded instead of ending the invocation early and using a later `await`-like function.\n\nHowever, for example, this can still be done with the built-in `Task` module (including to handle many API requests at once): \n\n    1..10\n    |> Enum.map(fn n ->\n      Task.async(fn ->\n        Tornex.Scheduler.Bucket.enqueue(pid, query)\n      end)\n    end)\n    |> Task.await_many(timeout)","ref":"Tornex.Scheduler.Bucket.html#module-making-api-requests"},{"type":"function","title":"Tornex.Scheduler.Bucket.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Tornex.Scheduler.Bucket.html#child_spec/1"},{"type":"function","title":"Tornex.Scheduler.Bucket.enqueue/2","doc":"Enqueues a `Tornex.Query` or `Tornex.SpecQuery` to the queue of the `Tornex.Scheduler.Bucket` for the API key's user.\n\nThe `pid` of the bucket belonging to the API key's user will be retrieved with `Tornex.Scheduler.Bucket.get_by_id/1`, and will be used to enqueue the query for that specific bucket.","ref":"Tornex.Scheduler.Bucket.html#enqueue/2"},{"type":"function","title":"Options - Tornex.Scheduler.Bucket.enqueue/2","doc":"* `:timeout` - Timeout of the GenServer call in milliseconds (default: `60_000`)","ref":"Tornex.Scheduler.Bucket.html#enqueue/2-options"},{"type":"function","title":"Tornex.Scheduler.Bucket.get_by_id/1","doc":"Retrieves the `pid` of the bucket by the bucket's user ID.\n\nUsing the `Tornex.Scheduler.BucketRegistry` that creates the relationship between the bucket and the user ID upon bucket creation, the `pid` of the bucket will be returned. If a bucket is not registered for that user, `:error` will be returned.","ref":"Tornex.Scheduler.Bucket.html#get_by_id/1"},{"type":"function","title":"Tornex.Scheduler.Bucket.new/1","doc":"Attempts to start a `Tornex.Scheduler.Bucket` and return the `pid` of a bucket.\n\nThe `Tornex.Scheduler.Supervisor` will attempt to start the bucket as a child. If the bucket does not exist, the bucket will be created and the `pid` of the bucket will be returned. If the bucket is already a child of `Tornex.Scheduler.Supervisor`, the `pid` of the existing bucket will be returned.\n\nIf there is an error in retrieving the `pid` of the bucket or an error in creating the bucket (e.g. maximum number of children in the supervisor reached), `nil` will be returned.","ref":"Tornex.Scheduler.Bucket.html#new/1"},{"type":"function","title":"Examples - Tornex.Scheduler.Bucket.new/1","doc":"iex> Tornex.Scheduler.Bucket.new(2383326)\n    #PID<0.105.0>","ref":"Tornex.Scheduler.Bucket.html#new/1-examples"},{"type":"function","title":"Tornex.Scheduler.Bucket.start_link/1","doc":"Starts the bucket for a user.","ref":"Tornex.Scheduler.Bucket.html#start_link/1"},{"type":"function","title":"Options - Tornex.Scheduler.Bucket.start_link/1","doc":"* `:user_id` - (any) A required unique identifier for the user the bucket belongs to","ref":"Tornex.Scheduler.Bucket.html#start_link/1-options"},{"type":"function","title":"Examples - Tornex.Scheduler.Bucket.start_link/1","doc":"iex> Tornex.Scheduler.Bucket.start_link(user_id: 2383326)\n    {:ok, process}\n    iex> is_pid(process)\n    true","ref":"Tornex.Scheduler.Bucket.html#start_link/1-examples"},{"type":"module","title":"Tornex.Scheduler.Supervisor","doc":"Default supervisor to supervise `Tornex.Scheduler.Bucket`, the dump timer, and the bucket registry.\nThe `Supervisor` can be replaced if necessary to modify bucket storage, the dump timer, etc.","ref":"Tornex.Scheduler.Supervisor.html"},{"type":"function","title":"Tornex.Scheduler.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Tornex.Scheduler.Supervisor.html#child_spec/1"},{"type":"function","title":"Tornex.Scheduler.Supervisor.start_link/1","doc":"Starts the `DynamicSupervisor` required for enqueuing a `Tornex.Query`.","ref":"Tornex.Scheduler.Supervisor.html#start_link/1"},{"type":"module","title":"Tornex.Scheduler.Timer","doc":"Timer used to determine when to dump all of the buckets, see `Tornex.Scheduler.Bucket`.\nBy default, the bucket will be dumped every 15 seconds to allow for a maximum call rate of \n60 API requests per minute. Upon the dump timer ending, the GenServer will send a `:dump` signal to \nall `Tornex.Scheduler.Bucket` under `Tornex.Scheduler.Supervisor`.","ref":"Tornex.Scheduler.Timer.html"},{"type":"function","title":"Tornex.Scheduler.Timer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Tornex.Scheduler.Timer.html#child_spec/1"},{"type":"function","title":"Tornex.Scheduler.Timer.start_link/1","doc":"Starts the `GenServer` for the `Tornex.Scheduler.Bucket` dump timer.","ref":"Tornex.Scheduler.Timer.html#start_link/1"},{"type":"module","title":"Tornex.SpecQuery","doc":"The query struct containing API request data for API v2 requests.\n\nThe `Tornex.SpecQuery` struct works against the OpenAPI specification for the Torn API v2. `Tornex.SpecQuery` can be used by `Tornex.API.get/1` and `Tornex.Scheduler.Bucket.enqueue/1` to make API calls. The struct stores the required information to perform the API call. To use APIv2 calls with the OpenAPI specification and an autogenerated client, you will need to add the [Tornium/torngen_elixir_client](https://github.com/Tornium/torngen_elixir_client) library to your dependencies; this library includes the paths and schemas required to make and parse APIv2 calls. Alternatively, to make APIv2 calls without the OpenAPI specification, use `Tornex.Query` instead and prepend the resource with `v2/` (e.g. `v2/user`).","ref":"Tornex.SpecQuery.html"},{"type":"module","title":"Preparing Query - Tornex.SpecQuery","doc":"Create an empty query using `new/0`. Use `put_path/2` and `put_parameter` to set up the query.\n\n    iex> query =\n    ...>   Tornex.SpecQuery.new(nice: 0)\n    ...>   |> Tornex.SpecQuery.put_key(\"foo\")\n    ...>   |> Tornex.SpecQuery.put_path(Torngen.Client.Path.User.Id.Bounties)\n    ...>   |> Tornex.SpecQuery.put_parameter(:id, 2383326)","ref":"Tornex.SpecQuery.html#module-preparing-query"},{"type":"module","title":"Making Query - Tornex.SpecQuery","doc":"The `SpecQuery` can be executed using `Tornex.API.get/1` and `Tornex.Scheduler.Bucket.enqueue/1` to perform the API calls. Once the API call has been finished, the response can be parsed into schema structs and validated using `parse/2`.\n\n    iex> api_response = Tornex.API.get(query)\n    iex> parsed_response = Tornex.SpecQuery.parse(query, api_response)","ref":"Tornex.SpecQuery.html#module-making-query"},{"type":"function","title":"Tornex.SpecQuery.new/1","doc":"Initialize an empty query against the OpenAPI specification.\n\nBy default, the niceness of the request will be set to 20 and the key owner will be set to 0. The key owner of 0 is intended to be for requests where the owner of the key is not known (e.g. for determining the owner of the API key).","ref":"Tornex.SpecQuery.html#new/1"},{"type":"function","title":"Options - Tornex.SpecQuery.new/1","doc":"* `:paths` - list of API paths\n  * `:parameters` - list of query/path parameters\n  * `:key` - API key\n  * `:key_owner` - ID of the owner of the API key (default: `0`)\n  * `:nice` - Priority of the API call between -20 and 20 (default: `20`)","ref":"Tornex.SpecQuery.html#new/1-options"},{"type":"type","title":"Tornex.SpecQuery.parameter/0","doc":"","ref":"Tornex.SpecQuery.html#t:parameter/0"},{"type":"function","title":"Tornex.SpecQuery.parse/2","doc":"Parse the response of an API call against the query into schema structs.","ref":"Tornex.SpecQuery.html#parse/2"},{"type":"function","title":"Tornex.SpecQuery.path_selections!/1","doc":"Parse a `SpecQuery` to determine the base path and selections.\n\nRaises `RuntimeError` if the `SpecQuery` does not contain any paths or has conflicting base paths.","ref":"Tornex.SpecQuery.html#path_selections!/1"},{"type":"function","title":"Examples - Tornex.SpecQuery.path_selections!/1","doc":"iex> query = \n    ...>   Tornex.SpecQuery.new()\n    ...>   |> Tornex.SpecQuery.put_path(Torngen.Client.Path.User.Id.Personalstats)\n    ...>   |> Tornex.SpecQuery.put_path(Torngen.Client.Path.User.Id.Bounties)\n    ...>   |> Tornex.SpecQuery.put_parameter(:id, 2383326)\n    ...>   |> Tornex.SpecQuery.put_parameter(:stats, [\"attackswon\", \"attackslost\"])\n    iex> Tornex.SpecQuery.path_selections!()\n    {\"user/{id}/\", [\"personalstats\", \"bounties\"]}\n\n    iex> query = \n    ...>   Tornex.SpecQuery.new()\n    ...>   |> Tornex.SpecQuery.put_path(Torngen.Client.Path.User.Id.Personalstats)\n    ...>   |> Tornex.SpecQuery.put_path(Torngen.Client.Path.User.Attacks)\n    ...>   |> Tornex.SpecQuery.put_parameter(:id, 2383326)\n    ...>   |> Tornex.SpecQuery.put_parameter(:stats, [\"attackswon\", \"attackslost\"])\n    iex> Tornex.SpecQuery.path_selections!()\n    ** (RuntimeError) 2 base paths were added to the query **\n\n    iex> query = Tornex.SpecQuery.new()\n    iex> Tornex.SpecQuery.path_selections!()\n    ** (RuntimeError) No paths were added to the query **","ref":"Tornex.SpecQuery.html#path_selections!/1-examples"},{"type":"function","title":"Tornex.SpecQuery.put_key/2","doc":"Add an API key to the query.","ref":"Tornex.SpecQuery.html#put_key/2"},{"type":"function","title":"Tornex.SpecQuery.put_key_owner/2","doc":"Add the owner of the API key to the query.","ref":"Tornex.SpecQuery.html#put_key_owner/2"},{"type":"function","title":"Tornex.SpecQuery.put_parameter/3","doc":"Add an OpenAPI parameter to the query.\n\nDuring the execution of the query, the parameter will be inserted as either a path or query parameter depending on the OpenAPI specifications for the paths included in the query. The `parameter_value` must be an implementation of the `String.Chars` protocol. If `parameter_value` is a list and the `parameter_name` is a query parameter, the values of the list will be joined with commas and used as the parameter value.","ref":"Tornex.SpecQuery.html#put_parameter/3"},{"type":"function","title":"Examples - Tornex.SpecQuery.put_parameter/3","doc":"iex> query = \n    ...>   Tornex.SpecQuery.new()\n    ...>   |> Tornex.SpecQuery.put_path(Torngen.Client.Path.User.Id.Personalstats)\n    ...>   |> Tornex.SpecQuery.put_parameter(:id, 2383326)\n    ...>   |> |> Tornex.SpecQuery.put_parameter(:stat, [\"attackswon\", \"attackslost\"])","ref":"Tornex.SpecQuery.html#put_parameter/3-examples"},{"type":"function","title":"Tornex.SpecQuery.put_path/2","doc":"Add an OpenAPI path to the query.\n\nThe OpenAPI path must be an implementation of the `Torngen.Client.Path` behavior (originating from the `torngen` library). However, multiple base resources can not be combined into one query when adding paths to the query. This includes paths for the same resource type (e.g. user) but differing resource IDs such as `/user/${id}/bounties`.","ref":"Tornex.SpecQuery.html#put_path/2"},{"type":"function","title":"Examples - Tornex.SpecQuery.put_path/2","doc":"iex> query =\n    ...>   Tornex.SpecQuery.new()\n    ...>   |> Tornex.SpecQuery.put_path(Torngen.Client.Path.User.Attacks)\n    ...>   |> Tornex.SpecQuery.put_path(Torngen.Client.Path.User.Bounties)","ref":"Tornex.SpecQuery.html#put_path/2-examples"},{"type":"type","title":"Tornex.SpecQuery.t/0","doc":"","ref":"Tornex.SpecQuery.html#t:t/0"},{"type":"function","title":"Tornex.SpecQuery.uri!/1","doc":"Generate and validate a URI to perform the `Tornex.SpecQuery`.\n\nRaises `RuntimeError` if the `SpecQuery` does not contain all of the necessary path parameters or if a base path and selections cannot be determined by `path_selections!/1`.","ref":"Tornex.SpecQuery.html#uri!/1"},{"type":"function","title":"Examples - Tornex.SpecQuery.uri!/1","doc":"iex> query = \n    ...>   Tornex.SpecQuery.new()\n    ...>   |> Tornex.SpecQuery.put_path(Torngen.Client.Path.User.Id.Personalstats)\n    ...>   |> Tornex.SpecQuery.put_path(Torngen.Client.Path.User.Id.Bounties)\n    ...>   |> Tornex.SpecQuery.put_parameter(:id, 2383326)\n    ...>   |> Tornex.SpecQuery.put_parameter(:stat, [\"attackswon\", \"attackslost\"])\n    iex> Tornex.SpecQuery.uri!(query)\n    %URI{\n      scheme: \"https\",\n      userinfo: nil,\n      host: \"api.torn.com\",\n      port: 443,\n      path: \"/v2/user/2383326\",\n      query: \"selections=personalstats,bounties&stat=attackswon,attackslost\",\n    }\n\n    iex> query = \n    ...>   Tornex.SpecQuery.new()\n    ...>   |> Tornex.SpecQuery.put_path(Torngen.Client.Path.Faction.Id.Chain)\n    ...>   |> Tornex.SpecQuery.put_path(Torngen.Client.Path.Faction.Id.Members)\n    iex> Tornex.SpecQuery.uri!(query)\n    ** (RuntimeError) Invalid fragment \"{id}\" in generated URI: https://api.torn.com/v2/faction/{id}/?selections=chain,members **","ref":"Tornex.SpecQuery.html#uri!/1-examples"},{"type":"module","title":"Tornex.Telemetry","doc":"Telemetry for Tornex using `:telemetry`.\n\n`Tornex.Telemetry` defaults to using `Logger` to handle telemetry events, but this behavior can be \ncustomized by not attaching the default logger.","ref":"Tornex.Telemetry.html"},{"type":"function","title":"Tornex.Telemetry.attach_default_logger/1","doc":"Attaches the default Tornex `:telemetry` handler built on-top of `Logger`.","ref":"Tornex.Telemetry.html#attach_default_logger/1"},{"type":"function","title":"Tornex.Telemetry.detach_default_logger/0","doc":"Detaches the default Tornex telemetry handler.","ref":"Tornex.Telemetry.html#detach_default_logger/0"},{"type":"extras","title":"Tornex","doc":"# Tornex\nAn Elixir library providing robust API call planning and execution for the [Torn City](https://torn.com) API.\n\n> #### NOTE\n> Currently, tornex is intended to work on single-node deployments. Multi-node deployments will experience issues with ratelimiting.","ref":"readme.html"},{"type":"extras","title":"Features - Tornex","doc":"- Efficient API ratelimiting and planning using buckets and prioritized requests\n- `:telemetry`-based logging (with included support for [PromEx](https://hex.pm/packages/prom_ex))\n- Support for APIv1 and APIv2","ref":"readme.html#features"},{"type":"extras","title":"Roadmap - Tornex","doc":"- [ ] Per-IP/node global ratelimit\n- [ ] Combine similar queries to reduce throughput\n- [ ] Multi-node support\n- [ ] APIv2 struct coalescing using [torngen](https://github.com/Tornium/torngen)\n- [ ] APIv2 mocking server using [torngen](https://github.com/Tornium/torngen)","ref":"readme.html#roadmap"},{"type":"extras","title":"Installation - Tornex","doc":"Once [available in Hex](https://hex.pm/docs/publish), the library can be installed\nby adding `tornex` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:tornex, \"~> 0.2.0\"}\n  ]\nend\n```\n\nFor latest changes, you can also install the library directly from [GitHub](https://github.com/Tornium/tornex):\n\n```elixir\ndef deps do\n  [\n    {:tornex, github: \"Tornium/tornex\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Tornex","doc":"Add the supervisor `Tornex.Scheduler.Supervisor` to your application supervisor. If you are using the default telemetry handler using `Tornex.Telemetry.attach_default_handler`, make sure to start the handler before starting the supervisor.\n\n<!-- tabs-open -->","ref":"readme.html#usage"},{"type":"extras","title":"APIv1 - Tornex","doc":"Create a `Tornex.Query` struct containing the request information:\n```elixir\nrequest = %Tornex.Query{\n    resource: \"user\",\n    resource_id: 2383326,\n    key: api_key,\n    selections: [\"attacks\", \"basic\"],\n    key_owner: 2383326,\n    nice: 10\n}\n```\n\nThe query struct must contain the following values:\n- `resource`: The name of the resource.\n- `resource_id`: The ID of the resource even if the request is against the key owner's user, faction, etc.\n- `key`: The API key to be used.\n- `key_owner`: Any other unique identifier (suggested to use Torn ID) of the API key owner.\n- `nice`: The priority of the request (following the Linux niceness values) where -20 is the highest priority and 19 is the lowest priority.","ref":"readme.html#apiv1"},{"type":"extras","title":"APIv2 - Tornex","doc":"To use the auto-generated client for APIv2, you will need to add the [Tornium/torngen_elixir_client](https://github.com/Tornium/torngen_elixir_client) library to your dependencies. Without installing this client, you will have to use the APIv1 style requests through `Tornex.Query` and prepend `v2` to the resource (e.g. `v2/user`).\n\nCreate a `Tornex.SpecQuery` struct containing the request information:\n```elixir\nrequest =\n    Tornex.SpecQuery.new()\n    |> Tornex.SpecQuery.put_path(Torngen.Client.Path.User.Attacks)\n    |> Tornex.SpecQuery.put_path(Torngen.Client.Path.User.Basic)\n    |> Tornex.SpecQuery.put_key(api_key)\n```\n<!-- tabs-close -->\n\nOnce the query struct has been constructed, the request can be enqueued into the key owner's queue with `Tornex.Scheduler.Bucket.enqueue/1` to be made depending on the request's priority and the state of the key owner's queue. Alternatively, the request can be performed immediately with `Tornex.API.get/1` which performs no ratelimiting. However, both methods are blocking.\n\n```elixir\nresponse = Tornex.Scheduler.Bucket.enqueue(request)\n\ncase response do\n    {:error, :timeout} ->\n        IO.puts(\"timeout\")\n\n    {:error, _} ->\n        IO.puts(\"unknown\")\n\n    %{\"name\" => name, \"attacks\" => attacks} ->\n        IO.puts(\"#{name} => #{Enum.count(attacks)} attacks\")\nend\n```","ref":"readme.html#apiv2"},{"type":"extras","title":"Documentation - Tornex","doc":"Documentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc) and can be found at  .","ref":"readme.html#documentation"},{"type":"extras","title":"License - Tornex","doc":"Copyright 2024-2025 tiksan\n\nThis project is licensed under Apache 2.0; see [LICENSE.md](LICENSE.md) for more details.","ref":"readme.html#license"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright [yyyy] [name of copyright owner]\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}